%%% *************************************************************
%%% Copyright (C) 2009 Torsten Anders (www.torsten-anders.de) 
%%% This program is free software; you can redistribute it and/or
%%% modify it under the terms of the GNU General Public License
%%% as published by the Free Software Foundation; either version 2
%%% of the License, or (at your option) any later version.
%%% This program is distributed in the hope that it will be useful,
%%% but WITHOUT ANY WARRANTY; without even the implied warranty of
%%% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%%% GNU General Public License for more details.
%%% *************************************************************

%%
%% Microtonal Helmholtz-Ellis notation with Lilypond
%%

%%
%% INSTALLATION
%%
%% - Install the font HE available at
%%   http://music.calarts.edu/~msabat/ms/pdfs/HE-font-2009.zip For
%%   example, on MacOS you can drop the file HE.ttf into
%%   /Library/Fonts
%%
%% - Add the content of this file to your Lilypond source file (e.g.,
%%   either directly or with \include)
%%
%%
%% USE
%%
%% Notate the score as you see in the example provided with this
%% file. There are two cases to distinguish: plain notes and notes in
%% a chord. Plain notes are notated with
%%
%%    \HE <string> <note>
%%
%% where <string> is an arbitrary string of HE font characters, and
%% <note> is a Lilypond note. The following example notates a B flat
%% transposed down by a septimal comma.
%%
%%    \HE "<e" b'
%%
%% Refer to the file HE-font-map.pdf provided with the HE font for the
%% mapping of characters to accidentals. Chord notes are instead
%% notated with \chordHE (e.g., \chordHE "<e" b')
%%
%% Note that this code completely disables the common Lilypond
%% accidentals, because the spacing (and to some extend also the
%% shape) of these and the HE accidentals differ considerably. In
%% order to allow for the common Lilypond accidentals anyway, comment
%% the three Lilypond code lines in the score layout section that set
%% accidentals to be written written as markups by default and sets
%% the default HE accidental.
%%


%%
%% Acknowledgments: 
%%
%% The help of Robin Bannister was invaluable fine-tuning the notation
%% created.  In particular, he contributed Lilypond code snippets that
%% care for the spacing of the accidentals (i.e. the accidental's
%% X-extent is now set to the X-extent of the respective markup of the
%% HE string, and additionally the accidentals are slightly moved to
%% the right closer to the notes they belong to. He also implemented
%% the function chordHE.
%%
%% This project makes also use of previous work related to accidental
%% tuning with Lilypond. Graham Breed proposed how to use markups as
%% accidentals. Mark Polesky showed how to change accidentals for
%% individual notes in a chord.
%%

\version "2.12.00"

%% Font HE put in as 2nd font, which can be accessed as \sans.
\paper{
#(define fonts (make-pango-font-tree "Century Schoolbook L"
                                     "HE"
                                     "Bitstream Vera Sans Mono"
                        1))
}

%% accessing the accidental of an individual note in a
%% chord and setting some properties (prop-alist)
#(define (modify-accidental note-grob prop-alist)
  ;; notehead before-line-breaking callback
  (let ((accidental (ly:grob-object note-grob 'accidental-grob)))
   (if (not (null? accidental))
    (for-each
     (lambda (x)
      (ly:grob-set-property! accidental (car x) (cdr x)))
     prop-alist))))

%% get the X-extend of a markup used as accidental
#(define (markup-X-extent markup)
  (lambda (grob)
   ;; slightly move the accidental combination to the right 
   ;; (the font introduces a gap before each sign)
   (interval-translate 
    ;; get the extend (horizontal size) of the accidental combination
    (ly:stencil-extent (grob-interpret-markup grob markup) X)   
    -0.5
  )))

%% setting the text of a chord accidental to a given markup
#(define (accidental-text markup)
  (lambda (grob)
   (modify-accidental grob   
    `(; show the markup extend as box (for debugging)
      ; (stencil . ,(lambda (grob) (box-stencil (ly:text-interface::print grob) 0 0)))
      (stencil . ,ly:text-interface::print)
      (text . ,markup)       
      (X-extent . ,(markup-X-extent markup))))))

%% Sets accidental of next note to a markup created with the given string using the HE font.
HE = 
#(define-music-function (parser location marktext) (string?)
  #{
  %% all accidentals are written as markups
  \once \override Accidental #'stencil = #ly:text-interface::print
  %% show the markup extend as box (for debugging)
%   \once \override Accidental #'stencil = #(lambda (grob) (box-stencil (ly:text-interface::print grob) 0 0))
  \once \override Accidental #'text = \markup { \sans $marktext }
  \once \override Accidental #'X-extent = #(markup-X-extent (markup #:sans $marktext))
  #})

%% expects a markup and sets accidental of next note to that markup
markupHE =
#(define-music-function (parser location markup) (markup?)
  #{
  %% all accidentals are written as markups
  \once \override Accidental #'stencil = #ly:text-interface::print
  %% show the markup extend as box (for debugging)
%   \once \override Accidental #'stencil = #(lambda (grob) (box-stencil (ly:text-interface::print grob) 0 0))
  \once \override Accidental #'text = #$markup
  \once \override Accidental #'X-extent = #(markup-X-extent $markup)
  #})

%% Sets accidental of next chord note to a markup created with the given string using the HE font.
%% markup. \override does not work for chords nor for accidentals, so
%% a special technique is required
chordHE =
#(define-music-function (parser location marktext mus) (string? ly:music?)  
  (set! (ly:music-property mus 'tweaks)        
   (acons 'before-line-breaking
    (accidental-text (markup #:sans marktext))               
    (ly:music-property mus 'tweaks)))
  mus) 

%% expects a markup and sets accidental of next chord note to that markup
markupChordAccidental =
#(define-music-function (parser location mkup mus) (markup? ly:music?)  
  (set! (ly:music-property mus 'tweaks)        
   (acons 'before-line-breaking (accidental-text mkup)               
    (ly:music-property mus 'tweaks)))
  mus) 


%% accidental rule set outside score so it can be defined in \layout
dodecaphonic = #`(Staff ,(lambda (c p bn mp) '(#f . #t)))

\layout {
  \context 
  { \Score
    %% there is an accidental for every note 
    autoAccidentals = #dodecaphonic
    autoCautionaries = #dodecaphonic
    %% show the markup extend as box (for debugging)
%     \override Accidental #'stencil = #(lambda (grob) (box-stencil (ly:text-interface::print grob) 0 0))
    %% by default all accidentals are written as markups
    \override Accidental #'stencil = #ly:text-interface::print
    %% default is the natural accidental
    \override Accidental #'text = #(markup #:sans "n")   
    \override Accidental #'X-extent = #(markup-X-extent (markup #:sans "n"))  
    %% more space between all notes 
%     \override SpacingSpanner #'base-shortest-duration = #(ly:make-moment 1 16)
  }
}


